

import { KEY_PREFIX } from './constants';

export default function getStoredState(config, onComplete) {
  var transforms = config.transforms || [];
  var storageKey = '' + (config.keyPrefix !== undefined ? config.keyPrefix : KEY_PREFIX) + config.key;
  var storage = config.storage;
  var debug = config.debug;

  var restoredState = {};
  var completionCount = 0;

  storage.getItem(storageKey, function (err, serialized) {
    if (err) {
      if (process.env.NODE_ENV !== 'production' && debug) console.log('redux-persist/getStoredState: Error in storage.get', err);
      onComplete(err);
    }

    if (!serialized) onComplete(null, undefined);else {
      try {
        var state = {};
        var rawState = deserialize(serialized);
        Object.keys(rawState).forEach(function (key) {
          state[key] = transforms.reduceRight(function (subState, transformer) {
            return transformer.out(subState, key);
          }, deserialize(rawState[key]));
        });
        onComplete(null, state);
      } catch (err) {
        if (process.env.NODE_ENV !== 'production' && debug) console.log('redux-persist/getStoredState: Error restoring data ' + serialized, err);
        onComplete(err);
      }
    }
  });

  return !!Promise && new Promise(function (resolve, reject) {
    onComplete = function onComplete(err, restoredState) {
      if (err) reject(err);else resolve(restoredState);
    };
  });
}

function deserialize(serial) {
  return JSON.parse(serial);
}